%% Counting machine iGEM 2005 ETHZ
% https://2006.igem.org/wiki/index.php/ETH_Zurich_2005

clear all;
close all; 

col = [255 59 48; 255 204 0; 76 217 100; 90 200 250; 255 149 0]/255; % User-predefined colors
style = {'-'; '- -'; '-.'; ':'};


n  = 4;         % Cooperativity of the repressor proteins
nSa  = 4;       % Cooperativity of the signal molecules when they act as activatora
nSr  = 4;       % Cooperativity of the signal molecules when they act as repressora
K  = 30;        % Affinity of the repressor proteins
KSa  = 0.1;     % Affinity of the signal molecules when they act as activators
KSr  = 0.1;     % Affinity of the signal molecules when they act as repressora
k0 = 0;         % Basal synthesis rate
k  = 3000;      % Induced maximal synthesis rate
kd  = 10;       % Protein degradation rate
Smax = 300;     % Signal amplitude
period = 40;    % Signal period
xlim0 = 160;    % Final time 

%n<-4; nSa<-4; nSr<- 4; K<- 30; KSa<- 0.1; KSr<-0.1; k<-3000; 
%kd<-10; Smax<-300; p<-50; t0<-0

para = [n,nSa,nSr,K,KSa,KSr,k0,k,kd,Smax,period];
y0 = [1 0 0 0];
opt = odeset('RelTol',1e-6,'AbsTol',1e-9); 

% stepsize
tstep = 0.01;
% duration

tfinal = xlim0;
tspan=[0 tfinal];

% run computation
[t,y] = ode45(@counter,0:tstep:tfinal,y0,opt,para);

s = signal_ode(Smax,0:tstep:tfinal,period);

    
%colors = ['k' 'k' 'k' 'k' 'k']; %for doing powerpoint presentations
% for i = 1:4;
%    subplot(5,1,i); 
%    plot(t,y(:,i),colors(i));
%    axis([0 tfinal 0 100]);
%    ylabel(sprintf('[R%d]', i))
%    hold on; 
% end
% subplot(5,1,5);
% plot(t,s,colors(5));
% axis([0 tfinal -0.1 max(s)*1.1]);
% ylabel('[S]')
% xlabel('time')

h_FSM = figure;
hold on;
for i = 1:6
    if i>4 %for proper use of color vector (it only contains four colors)
    ii=i-4;
else 
    ii=i;
end
plot(t,y(:,ii),'Color',col(ii,:), 'LineStyle', style{1});
end

plot(t,s,'Color','k'); 
xlim([0 xlim0]); ylim([0 1.2*max(max(y))]);
hold off;

ax = gca; ax.LineWidth = 2; ax.FontSize = 16; ax.FontWeight = 'bold'; ax.XColor = 'k'; ax.YColor = 'k';
ax.TickDir = 'in'; ax.Clipping = 'on'; ax.ClippingStyle = 'rectangle'; ax.Layer = 'Top';
ax.XAxis.TickLabelFormat = '%.0f'; ax.YAxis.TickLabelFormat = '%.0f';
set(0, 'DefaultLineLineWidth', 2); box on;

xlabel('Time (min)'); ylabel('Repressor concentration (arb. units)');

set(gcf,'units','pixels','position',[0,0,700,600]);    
%print(h_FSM,'C:\Users\...\FSM_plot','-dpng','-r500')
%%
opt = odeset('RelTol',1e-6,'AbsTol',1e-9); 

n  = 2;  
nSa  = 2;
nSr  = 2;
K  = 30; 
KSa  = 0.1;
KSr  = 0.1;
k  = 3000;
k0 = 0;
kd  = 0.02; 
Smax = 100;
period = 40;
xlim0 = 1600;
xlim1 = 3200; 

tstep = 1;
tfinal = xlim1;
tspan=[0 tfinal];

para = [n,nSa,nSr,K,KSa,KSr,k0,k,kd,Smax,period];
y0 = [1 0 0 0];

[t,y] = ode45(@counter,0:tstep:tfinal,y0,opt,para);



s = signal_ode(Smax,0:tstep:tfinal,period);

h_FSM2 = figure;
hold on;
loglog(t,y);
for i = 1:4;
plot(t,y(:,i),'Color',col(i,:), 'LineStyle', style{1}); 
end

n_steps = 22;
for i = 1:n_steps;
i0 = (1780-xlim0)+1600;
i1 = i0 + 40*(i-1);
if i < 8
ix = -40;
elseif i == 8
ix = -15;    
else
ix = 0;        
end
text(1780 + 40*(i-1)+ix,1.6*y(i1,1),num2str(i),'FontWeight','light','FontSize', 14, 'BackgroundColor', 'w','margin',0.00001);
plot(1780 + 40*(i-1),y(i1,1), 'o', 'Color','k');

end
text(1780 - 40-40,1.6*y(i0-40,1),num2str(0),'FontWeight','light','FontSize', 14, 'BackgroundColor', 'w','margin',0.00001);
plot(1780 - 40,y(i0-40,1), 'o', 'Color','k');


plot(t,s/Smax/1.3,'Color','k'); 
xlim([xlim0 xlim1]); 
ylim([0 1.1*max(max(y))]); set(gca,'YScale','log'); ylim([0.3 1.*10^5]); 
hold off;
nice_plot;
xlabel('Time (min)'); ylabel('Repressor concentration (arb. units)');


set(gcf,'units','pixels','position',[0,0,700,600]);    
%print(h_FSM2,'C:\Users\...\FSM_plot2','-dpng','-r500')

%%
col = [255 59 48; 255 204 0; 76 217 100; 90 200 250; 255 149 0]/255;
style = {'-'; '- -'; '-.'; ':'};
colors = ['r' 'y' 'g' 'b' 'k'];
textR = {'R1'; 'R2'; 'R3'; 'R4'};


n  = 2;  
nSa  = 2; 
nSr  = 2; 
K  = 30; 
KSa  = 10;
KSr  = 165;
k  = 3000;
k0 = 0;
kd  = 10; 
Smax = 300;
period = 40;
xlim0 = 25; 

tstep = 0.01;
tfinal = xlim0;
tspan=[0 tfinal];

para = [n,nSa,nSr,K,KSa,KSr,k0,k,kd,Smax,period];
y0 = [1 0 0 0];

[t,y] = ode45(@counter,0:tstep:tfinal,y0,opt,para);

s = signal_ode(Smax,0:tstep:tfinal,period);

h_FSM3 = figure;
hold on;
plot(t,s,'Color','k'); 
for i = 1:4;
plot(t,y(:,i),'Color',col(i,:), 'LineStyle', style{1}); 
end

xlim([0 xlim0]); 
ylim([0 150]); 
set(gca,'YScale','log'); ylim([2 3.5*10^2]); 
%set(gca,'XScale','log'); xlim([1 25]); 
hold off;
nice_plot;
xlabel('Time (min)'); ylabel('Repressor concentration (arb. units)');



set(gcf,'units','pixels','position',[0,0,700,600]);    
%print(h_FSM3,'C:\Users\...\FSM_plot3','-dpng','-r500')

function dy = counter(t,y,para)
        
        % Parameter values    
        
        n  = para(1); 
        nSa  = para(2); 
        nSr  = para(3); 
        K  = para(4); 
        KSa  = para(5);
        KSr  = para(6);
        k0 = para(7);
        k  = para(8); 
        kd  = para(9); 
        Smax = para(10);
        period = para(11);
        
        % Input signal
        S = signal_ode(Smax,t,period);
        
        dy1 = k0 + k * act(S,KSa,nSa) * rep(y(2),K,n) * rep(y(3),K,n) - kd * y(1); 
        dy2 = k0 + k * rep(S,KSr,nSr) * rep(y(3),K,n) * rep(y(4),K,n) - kd * y(2); 
        dy3 = k0 + k * act(S,KSa,nSa) * rep(y(4),K,n) * rep(y(1),K,n) - kd * y(3); 
        dy4 = k0 + k * rep(S,KSr,nSr) * rep(y(1),K,n) * rep(y(2),K,n) - kd * y(4);
        
                
        dy = [dy1;dy2;dy3;dy4];
        
              
end


function Y = signal_ode(Smax,x,Tx)
% Generates the input signal, i.e. defines the signal shape

    % Y = Smax*0.5*(1+sign(sin(x.*2.*pi./Tx))); % Rectangular signal (initially high), periode Tx
    % Y = Smax*0.5*(1-sign(sin(x.*2.*pi./Tx))); % Rectangular signal (initially low), periode Tx
    % Y = Smax*0.5*(1+(sin(x.*2.*pi./Tx)));     % Sine, periode Tx
    % Y = Smax*0.5*(1+sign(sin(x.*2.*pi./Tx))).*0.5.*(1+sign(sin(x.*2.*pi./(2*Tx))));  % Rectangular signal (with   delay between peaks), periode 2Tx
    % Y = Smax*0.5*(1+(sin((x-Tx/4).*2.*pi./Tx))).*0.5.*(1+sign(sin(x.*2.*pi./(2*Tx)))); % Sine-like pulse signal  (with delay between peaks), periode 2Tx
      Y = Smax.*sin(2.*pi./Tx.*x).*(Smax.*sin(2.*pi./Tx.*x)>=0); %Positive sinus, negative values become zero;
   
                        
end



function pFactor = act(a,K,n)

    % activation
    pFactor = (a./K).^n./(1+(a./K).^n);  % alternative equation
  
    %pFactor = K.*a.^n./(1+K.*a.^n);
      
    
end

function pFactor = rep(r,K,n)

    % Jrg's function:
    % r: concentration of repressor
    % K: Hill constant (Affinity)
    % n: cooperativity, i.e. how steep the curve will be (plot([0:0.05:3], rep([0:0.05:3], 1, 5)))
   
    pFactor = 1./(1+(r./K).^n); 
    
    %pFactor = 1./(1+K.*r.^n);
    
end


function nice_plot()

ax = gca; ax.LineWidth = 2; ax.FontSize = 16; ax.FontWeight = 'bold'; ax.XColor = 'k'; ax.YColor = 'k';
ax.TickDir = 'in'; ax.Clipping = 'on'; ax.ClippingStyle = 'rectangle'; ax.Layer = 'Top';
ax.XAxis.TickLabelFormat = '%.0f'; ax.YAxis.TickLabelFormat = '%.1f';
set(0, 'DefaultLineLineWidth', 2); box on;
set(gcf,'units','pixels','position',[0,0,700,600]);    

end