%% Finite State Machine - Gillespie algorithm

clear all; close all; 

srand=1000*rand(1);
rng(srand,'philox');
col = [255 59 48; 255 204 0; 76 217 100; 90 200 250; 255 149 0]/255;

Trans = 0; %Initial time after which data is saved
Tend = 160;  %Final time
Tech = 0.01; %Step of time

Period = 40;
KSa  = 0.1;
KSr  = 0.1;
Omega = 10.^[-4:2:0];
k = 3000;
R1_init = 40;
%Omega = 10.^-1;
clear FSGM
FSMG=struct;
for i = 1:length(Omega)
%fprintf('Run simulation for Omega = %g \n', Omega(i));
D = runFSM_stoch(Omega(i),Trans, Tend, Tech, Period, KSa, KSr, k, R1_init);
%figure; h_FSMstoch=semilogy(D(:,1), D(:,2:5));
%set(h_FSMstoch, {'color'}, {col(1,:); col(2,:); col(3,:); col(4,:)});
%ylim([0 1.1*max(max(D(end-100:end,2:5)))]); xlim([0 20]);
FSMG(i).Data=D;


end
%%
figure; 
h_FSMstoch_1 = plot(FSMG(end).Data(:,1), FSMG(end).Data(:,2:5));
set(h_FSMstoch_1, {'color'}, {col(1,:); col(2,:); col(3,:); col(4,:)});


ax = gca; ax.LineWidth = 2; ax.FontSize = 16; ax.FontWeight = 'bold'; ax.XColor = 'k'; ax.YColor = 'k';
ax.TickDir = 'in'; ax.Clipping = 'on'; ax.ClippingStyle = 'rectangle'; ax.Layer = 'Top';
ax.XAxis.TickLabelFormat = '%.0f'; ax.YAxis.TickLabelFormat = '%.0f';
set(0, 'DefaultLineLineWidth', 2); box on;

xlabel('Time (min)'); ylabel('Repressor concentration (arb. units)');
set(gcf,'units','pixels','position',[0,0,700,600]);    

%%

function results=runFSM_stoch(omega, trans, tend, tech, period, KSa, KSr, k1, R1_init)

% Parameters

n  = 4;  
nSa  = 4; 
nSr  = 4; 
K  = 30; 
%KSa  = 0.1;
%KSr  = 0.1;
k  = k1*omega;
kd  = 10; 
Smax = 300;
%period = 40;

% Initial conditions:

R1 = R1_init; R2 = 0; R3 = 0; R4 = 0;
v = 1*[R1 R2 R3 R4]; 


% Initialisation:

t=0;
results=[];
told=0;

% Run Gillespie loop:

while (t<tend+trans)
S = signal_stoch(Smax,t,period,trans);

R1 = v(1);
R2 = v(2);
R3 = v(3);
R4 = v(4);
   
w(1) = k * ((S/KSa)^nSa)/(1 + (S/KSa)^nSa) * (omega^n)/(omega^n+(R2/K)^n) * (omega^n)/(omega^n+(R3/K)^n);
w(2) = k *           1/(1+(S/KSr)^nSr) * (omega^n)/(omega^n+(R3/K)^n) * (omega^n)/(omega^n+(R4/K)^n);
w(3) = k * ((S/KSa)^nSa)/(1 + (S/KSa)^nSa) * (omega^n)/(omega^n+(R4/K)^n) * (omega^n)/(omega^n+(R1/K)^n);
w(4) = k *           1/(1+(S/KSr)^nSr) * (omega^n)/(omega^n+(R1/K)^n) * (omega^n)/(omega^n+(R2/K)^n);
w(5) = kd * R1;
w(6) = kd * R2;
w(7) = kd * R3;
w(8) = kd * R4;

c(1)=w(1);
for j=2:8;
  c(j)=c(j-1)+w(j);
end
ct=c(8);

z1=rand();
z2=rand();

tau=(-log(z1))/ct;
uct=z2*ct;

tR1=0;
tR2=0;
tR3=0;
tR4=0;

if (uct<c(1))
  tR1 = 1.0;
elseif ((uct>c(1)) & (uct<c(2)))
  tR2 = 1.0;
elseif ((uct>c(2)) & (uct<c(3)))
  tR3 = 1.0;
elseif ((uct>c(3)) & (uct<c(4)))
  tR4 = 1.0;
elseif ((uct>c(4)) & (uct<c(5)))
  tR1 = -1.0;
elseif ((uct>c(5)) & (uct<c(6)))
  tR2 = -1.0;
elseif ((uct>c(6)) & (uct<c(7)))
  tR3 = -1.0;
elseif ((uct>c(7)) & (uct<c(8)))
  tR4 = -1.0;
end

tv=[tR1 tR2 tR3 tR4];

t=t+tau;

v=v+tv;

if (t>trans) && (t>told+tech)
   results = [results ; t-trans v];
   told=t;
end

end 



end



function Y = signal_stoch(Smax,x,Tx,Disp)
% Generates the input signal, i.e. defines the signal shape

    % Y = Smax*0.5*(1+sign(sin((x-Disp).*2.*pi./Tx))); % Rectangular signal (initially high), periode Tx
    % Y = Smax*0.5*(1-sign(sin((x-Disp).*2.*pi./Tx))); % Rectangular signal (initially low), periode Tx
    % Y = Smax*0.5*(1+(sin((x-Disp).*2.*pi./Tx)));     % Sine, periode Tx
    % Y = Smax*0.5*(1+sign(sin((x-Disp).*2.*pi./Tx))).*0.5.*(1+sign(sin((x-Disp).*2.*pi./(2*Tx))));  % Rectangular signal (with   delay between peaks), periode 2Tx
    % Y = Smax*0.5*(1+(sin(((x-Disp)-Tx/4).*2.*pi./Tx))).*0.5.*(1+sign(sin((x-Disp).*2.*pi./(2*Tx)))); % Sine-like pulse signal  (with delay between peaks), periode 2Tx
      Y = Smax.*sin(2.*pi./Tx.*(x-Disp)).*(Smax.*sin(2.*pi./Tx.*(x-Disp))>=0); %Positive sinus, negative values become zero;
                        
end
